<?php
/**
 * Classes of PHP PageBox
 * pagebox.inc
 * Version 0.1
 * Copyright (c) 2001-2002 Alexis Grandemange
 * Mail: alexis.grandemange@pagebox.net
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version
 * 2.1 of the License.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * A copy of the GNU Lesser General Public License lesser.txt should be
 * included in the distribution.
 */
/**
 * Write a log entry.
 */
function pblog($msg) {
  $flog = fopen("log.html", "a");
  if ($flog == FALSE)
    return FALSE;
  fwrite($flog, "<tr><td>" . date("d M Y h:i:s") . "</td><td>" .
    $msg . "</td></tr>\r\n");
  fclose($flog);
  return TRUE;
}
/**
 * SOAP client code to call the RepoQuery Web service.
 * Parameters:
 * pao: PbArchives instance
 * archive: archive whose we want the subscribers
 * deployURL: this deployment URL (to ignore in the list)
 * Returns an array of subscribers update.php URL (strings). 
 */
function repoQuery($pao, $archive, $deployURL) {
    include_once('SOAP/Client.php');
    if (($pao == NULL) || ($pao->archives == null))
        return NULL;
    if (!key_exists($archive, $pao->archives))
        return NULL;
    $url = $pao->archives[$archive]->download;
    $url = "http://" . str_replace("download", "repoQuery/RepoQuery.php", $url);
    pblog("Query.GetSubscribers(" . $archive . ", " . $deployURL .
        " invoked - RepoQuery at " . $url);
    $soapclient = new SOAP_Client($url);
    $ra = $soapclient->call("GetSubscribers", array("archive" => $archive,
        "deployURL" => $deployURL), 'urn:Repository');
    return $ra;
}
/**
 * Describes a PageBox archive.
 */
class Archive {
  /** Download location (repository) */
  var $download;
  /** Archive owner (publisher) */
  var $owner;
  /** Archive size */
  var $size;
  /** Archive last modified time */
  var $date;
  /** Archive documentation URL */
  var $doc;
  /**
   * Constructor.
   * Parameters:
   * l: Download location
   * o: Archive owner
   * s: Archive size
   * d: Archive last modified time
   * u: Archive documentation URL
   */
  function Archive($l, $o, $s, $d, $u) {
    $this->download = $l;
    $this->owner = $o;
    $this->size = $s;
    $this->date = $d;
    $this->doc = $u;
  }
}
/**
 * Describes deployed archives.
 * Serialized in pbArchives.pb.
 */
class PbArchives {
  /**
   * Archive array.
   * Key: archive name
   * Value: Archive object
   */
  var $archives;
  /**
   * Command used to inflate the archive.
   * 1 parameter: archive name.
   * Read from inflate.pb.
   */
  var $inflate;
  /**
   * Command used to remove an inflated archive.
   * 1 parameter: archive name.
   * Read from delete.pb.
   */
  var $delete;
  /**
   * Constructor.
   * Creates inflate.
   */
  function PbArchives() {
    if ($this->inflate != NULL) {
      return;
    }
    if (!file_exists("inflate.pb")) {
      return;
    }
    $fcmd = fopen("inflate.pb", "r");
    while ($iar = fgetcsv ($fcmd, 255, ":")) {
      if (($iar == NULL) || (count($iar) < 2)) {
        continue;
      }
      $this->inflate[trim($iar[0])] = trim($iar[1]);
    }
    fclose($fcmd);
    if ($this->delete != NULL) {
      return;
    }
    if (!file_exists("delete.pb")) {
      return;
    }
    $fcmd = fopen("delete.pb", "r");
    while ($iar = fgetcsv ($fcmd, 255, ":")) {
      if (($iar == NULL) || (count($iar) < 2)) {
        continue;
      }
      $this->delete[trim($iar[0])] = trim($iar[1]);
    }
    fclose($fcmd);
  }
  /**
   * Adds an archive.
   * Returns a status (String)
   * Parameters:
   * name: archive name
   * download: Download location
   * owner: Archive owner
   * size: Archive size
   * date: Archive last modified time
   * doc: Archive documentation URL
   */
  function add($name, $download, $owner, $size, $date, $doc) {
    if ($this->inflate == NULL) {
      return "Unsupported extension: inflate.pb not found";
    }
    $pieces = explode(".", $name);
    $inflate = $this->inflate[$pieces[1]];
    if ($inflate == NULL) {
      return "Unsupported extension: " . $pieces[1];
    }
    if ($this->archives[$name] != NULL) {
      $a = $this->archives[$name];
      if ($a->owner != $owner) {
        return "Invalid owner";
      } else {
        if (file_exists($name)) {
          unlink($name);
        }
        unset($this->archives[$name]);
      }
    }
    // Download the archive
    $farch = fopen('http://' . $download . "/" . $name, "br");
    if ($farch == FALSE) {
      return "http://" . $download . "/" . $name . " not found";
    }
    $rd = fread ($farch, $size);
    fclose($farch);
    $farch = fopen($name, "wb");
    if ($farch == FALSE) {
      return "Unable to write " . $name;
    }
    fwrite($farch, $rd, $size);
    fclose($farch);
    // Inflate the archive
    // echo $inflate . " " . $pieces[0];
    if ($inflate != "-") {
      exec($inflate . " " . $pieces[0]);
    }
    $status = $name . " installed";
    $a = new Archive($download, $owner, $size, $date, $doc);
    $this->archives[$name] = $a;
    return $status;
  }
  /**
   * Deletes an archive.
   * Returns a status (string).
   * Parameter:
   * name: archive name.
   */
  function delete($name) {
    if ($this->archives[$name] == NULL) {
      return $name . " is not installed";
    }
    unset($this->archives[$name]);
    // Inflate the archive
    if ($this->delete != NULL) {
      $pieces = explode(".", $name);
      $delete = $this->delete[$pieces[1]];
      if ($delete != NULL) {
        exec($delete . " " . $pieces[0]);
      }
    }
    unlink($name);
    return $name . " uninstalled";
  }
}
?>